package com.fahen.bluetooth_manager;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.os.Build;

/** BluetoothManagerPlugin */
public class BluetoothManagerPlugin implements FlutterPlugin, MethodCallHandler {
  private MethodChannel channel;

  private BluetoothAdapter mBluetoothAdapter;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "bluetooth_manager");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    String callResult = "";
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("getBluetoothState")) {
      callResult = getBluetoothState();
    } else if (call.method.equals("enableBluetooth")) {
      callResult = enableBluetooth();
    } else if (call.method.equals("disableBluetooth")) {
      callResult = disableBluetooth();
    } else {
      result.notImplemented();
    }

    if (callResult == "version_error" || callResult == "responseError") {
      result.error("ERROR", "Android version lower than 4.4 (KitKat) is unsupported | .", callResult);
    } else {
      result.success(callResult);
    }
  }

  private String getBluetoothState() {
    try {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
        final BluetoothManager bluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();
        if (mBluetoothAdapter.isEnabled()) {
          return "on";
        } else {
          return "off";
        }
      } else {
        return "version_error";
      }
    } catch (Exception e) {
      System.out.println(e);
      return "uknow";
    }
  }

  private String enableBluetooth() {
    try {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
        final BluetoothManager bluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();

        if (mBluetoothAdapter.isEnabled()) {
          return "bluetoothAlreadyOn";
        } else {
          mBluetoothAdapter.enable();
          return "bluetoothIsOn";
        }
      } else {
        return "version_error";
      }
    } catch (Exception e) {
      System.out.println(e);
      return "responseError";
    }
  }

  private String disableBluetooth() {
    try {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
        final BluetoothManager bluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();

        if (mBluetoothAdapter.isEnabled()) {
          mBluetoothAdapter.disable();
          return "bluetoothIsOff";
        } else {
          return "bluetoothAlreadyOff";
        }
      } else {
        return "version_error";
      }
    } catch (Exception e) {
      System.out.println(e);
      return "responseError";
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
